/*******************************************
Integration 1.0
Copyright 2010, Tiago Sommer Damasceno and
  The Shodor Education Foundation, Inc.
  
A C implementation of Area Under a Curve.

To run:
./Integration [Starting_Range] [Ending_Range] [Number of Segments]

See the README included in this directory for
more detailed information.
*******************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define xSquare 0
#define xFunc   1

#define left	0
#define right	1
#define mid		2


struct curve 
{
	long double dx, x, y, area, RANGE_START, RANGE_END;
	long long function, method_rule, SEGMENTS;	
};

void initialize(struct curve *curve){
	curve->dx 			= 0.0;
	curve->x 			= 0.0;
	curve->y 			= 0.0;	
	curve->area 		= 0.0;
	curve->function 	= xSquare;
	curve->method_rule 	= mid; 
	curve->SEGMENTS 	= 1000;
	curve->RANGE_START 	= 0;
	curve->RANGE_END 	= 1;
}

/*
	long double f();
		-	Adjusts X in order to Integrate using Left, Right or Mid Point Rule.
		RETURNS: Y Value for the X passed in.
*/
long double f(struct curve *curve){
    
	switch(curve->method_rule){
		case left:
			break;
		case right:
			curve->x = curve->x + curve->dx;
			break;
		case mid:
			curve->x = curve->x + (curve->dx*0.5);
			break;
		default:
			break;
	}
	
	switch(curve->function){
		case xSquare:
			curve->y = (curve->x)*(curve->x);
			break;
		case xFunc:
			curve->y = (curve->x);
			break;
		default:
			break;
	}   
}
/*
	void integrate()
		-	Computes dx based on segments, starting and ending range.
		-	Calls the f() function to compute Y values for each X.
		-	Calculates the area of each segment and adds them all.
*/
void integrate(struct curve *curve){
	
	long long count = 0;
	curve->dx = (curve->RANGE_END - curve->RANGE_START)/(long double)curve->SEGMENTS;

	
	for(count = 0; count < curve->SEGMENTS; count++){
		curve->x = count*curve->dx;
		f(curve);
		curve->area = curve->area + curve->y*curve->dx;
	}
}

void cleanup(struct curve *curve){
    curve->dx           = 0.0;
    curve->x            = 0.0;
    curve->y            = 0.0;
    curve->area         = 0.0;
    curve->function     = xSquare;
    curve->method_rule  = mid;
   // curve->SEGMENTS     = 1000;
   // curve->RANGE_START  = 0;
   // curve->RANGE_END    = 1;
}
/*
	int main()
		-	Initializes variables.
*/
int main(int argc, char **argv){
	
	struct curve curveOne; // Variable which contains all 
	
	initialize(&curveOne);

	if(argc < 4) exit(0);
	curveOne.RANGE_START = atoi(argv[1]);
	curveOne.RANGE_END 	 = atoi(argv[2]);
	curveOne.SEGMENTS 	 = atoi(argv[3]);

	if(curveOne.RANGE_END <= curveOne.RANGE_START || curveOne.SEGMENTS < 1) exit(0);
	
	curveOne.function 	 	= xSquare;

	curveOne.method_rule 	= left;
	integrate(&curveOne);
	printf("Left Point Rule  - Area: %Lf\n", curveOne.area);
	cleanup(&curveOne);	

	curveOne.method_rule	= mid;
	integrate(&curveOne);
	printf("Mid Point Rule   - Area: %Lf\n", curveOne.area);
	cleanup(&curveOne);

	curveOne.method_rule	= right;
	integrate(&curveOne);
	printf("Right Point Rule - Area: %Lf\n", curveOne.area);
	cleanup(&curveOne);	

	return(0);
}